# -*- coding: utf-8 -*-

'''
Device
============

È stata creata la classe Device estendendo la classe AbstractFCMDevice (di fcm-django) 
per ridefinirne alcuni attributi e per aggiungere funzionalità

* :class:`Device` (``AbstractFCMDevice``, ``DateModel``, ``OrderedModel``)


Modelli
========

.. autoclass:: Device
   :members:

'''
from __future__ import unicode_literals

from django.conf import settings
from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from fcm_django.models import AbstractFCMDevice
from firebase_admin.messaging import Message, Notification, WebpushConfig, WebpushFCMOptions

from thx.core.models import UserModel, DateModel, OrderedModel


class Device(AbstractFCMDevice, DateModel, OrderedModel):
    version = models.CharField(
        verbose_name=_("version"),
        max_length=150
    )
    manufacturer = models.CharField(
        verbose_name=_("manufacturer"),
        max_length=150
    )
    reference = models.CharField(
        blank=True, null=True,
        max_length=150,
        verbose_name=_('reference'),
    )

    class Meta:
        verbose_name = _('device')
        verbose_name_plural = _('devices')
        ordering = ('ordering', '-date_last_modify')
        constraints = [
            models.UniqueConstraint(fields=['device_id', 'registration_id'], name='unique device')
        ]
        permissions = (
            ("list_device", "Can list device"),
            ("detail_device", "Can detail device"),
            ("disable_device", "Can disable device"),
        )

    def __str__(self):
        return '{}: {}'.format(
            self.reference,
            self.device_id
        )

    @classmethod
    def get_test_data_for_push(cls):
        return {
            "text": "Test FCM send <strong> YES! </strong> test length test length test length test length.",
            "label": f"Test FCM send {timezone.now()}",
            "link": {
                "label": 'Go to your badges',
                "url": 'notification-center'
            },
            "display": False,
            "icon": 'statics/svg-icons/profile-trophy.svg'
        }

    @classmethod
    def get_test_notification_for_push(cls):
        return {
            "title": "Title: Test FCM send",
            "body": f"Body: Test FCM send {timezone.now()}",
            "image": "https://adk.it/statics/icons/android-icon-192x192.png",
            "icon": "https://adk.it/statics/icons/android-icon-192x192.png"
        }

    @classmethod
    def get_test_extra_option_for_push(cls):
        return {
            'webpush': {
                'fcm_options': {
                    'link': 'https://adrenalinik.quasar.thux.dev'
                },
                'headers': {
                    'Urgency': 'high'
                },
            },
            "priority": "high",
        }

    def send_test_push_message(self, data=None, notification=None, extra_option=None):
        if not data:
            data = self.get_test_data_for_push()
        if not notification:
            notification = self.get_test_notification_for_push()
        if not extra_option:
            extra_option = self.get_test_extra_option_for_push()
        # self.send_message(data=data, notification=notification, **extra_option)
        self.send_fcm_message(**notification,  **extra_option)

    def send_fcm_message(self, title, body, image=None, web_push_link=None, devices=None, *args, **kwargs):
        msg = Message(
            notification=Notification(
                title=title,
                body=body,
                image=image
            ),
            data={
                "title": title,
                "body": body,
                "icon": image,
                "image": image
            },
            webpush=WebpushConfig(
                fcm_options=WebpushFCMOptions(link=web_push_link))
        )
        if not devices:
            devices = Device.objects.filter(active=True)
        devices.send_message(msg)
